/* CubeGL.c */
/*Darstellung eines dreidimensionalen Wuerfels mit OpenGL*/
#include <conio.h>  //wg. _getch()
#include <stdio.h>  //wg. printf()
//#include <stdlib.h> //wg. system()
#include <GL/glut.h>

#define CON_CLS   "cls"
#define ESC        27

enum {X=0, Y=1, Z=2, W=3};

/*Globale Variablen: */
float    eyez=10., diag=0., viewScale=1., angle[3]={0.,0.,0.};
int      backF=0, persp=1, winWidth=300, winHeight=300; 
GLdouble nah=10., fern=10.;
GLfloat  Vrtx[8][3]; 
GLint    faces[6][4] = { {0, 1, 2, 3}, {1, 0, 4, 5}, {1, 5, 6, 2}, 
                         {4, 0, 3, 7}, {3, 2, 6, 7}, {5, 4, 7, 6}
                       };

/*************************************************************************/
  void drawBox(void)
/*************************************************************************/
{ int jj;
  nah=eyez; fern=nah+2*diag;

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity(); //Tabula rasa...

  if (persp) /*Perspektive:*/
  { /*Mittige Plazierung der mit viewScale skalierten Szene:*/
    glViewport(winWidth*(1-viewScale)/2, winHeight*(1-viewScale)/2, 
               winWidth*viewScale,       winHeight*viewScale); 
    glFrustum (-diag, diag, -diag, diag, nah, fern);
  }
  else /*Parallel-Projektion:*/
  { glViewport(0, 0, winWidth, winHeight); 
    glOrtho (-diag, diag, -diag, diag, nah, fern);
  }

  if (backF) glEnable  (GL_CULL_FACE);
  else       glDisable (GL_CULL_FACE);

  /*Bisherige Bewegung neu aufbauen (ohne Fehlerfortpflanzung):*/
  glMatrixMode(GL_MODELVIEW);

  /*Vorm Weiterrechnen aktuelle Modell-Positionierung merken:*/
  glPushMatrix (); 

  /*Ins Sichtvolumen verschieben:*/
  glTranslatef(0.0, 0.0, -(nah+diag/2));
  glRotatef(angle[X], 1.0, 0.0, 0.0);
  glRotatef(angle[Y], 0.0, 1.0, 0.0);
  glRotatef(angle[Z], 0.0, 0.0, 1.0);

  /*Wuerfel zeichnen:*/
  for (jj = 0; jj < 6; jj++)
  { glBegin(GL_POLYGON); // ebenso: glBegin(GL_QUADS);
    glVertex3fv(Vrtx[faces[jj][0]]);
    glVertex3fv(Vrtx[faces[jj][1]]);
    glVertex3fv(Vrtx[faces[jj][2]]);
    glVertex3fv(Vrtx[faces[jj][3]]);
    glEnd();
  }
  /*Letzte Berechnungen zu Modell-Positionierung vergessen:*/
  glPopMatrix (); 
}

/*************************************************************************/
  void draw(void)
/*************************************************************************/
{ glClear(GL_COLOR_BUFFER_BIT);
  drawBox();
  glutSwapBuffers();
}

/*************************************************************************/
  void key(unsigned char key, int x, int y)
/*************************************************************************/
/*Menue und Eingabe-Behandlung:*/
{ switch (key)
  { case ESC: exit(0);
    case 'b': backF = 1-backF;                break;
    case 'e': eyez *=.9f;  viewScale /= .9f;  break;
    case 'E': eyez *=1.1f; viewScale /= 1.1f; break;
    case 'p': persp = 1-persp;                break;
    case 'x': angle[X] -= 5.; if (angle[X] <=-360) angle[X]+=360; break;
    case 'X': angle[X] += 5.; if (angle[X] >= 360) angle[X]-=360; break;
    case 'y': angle[Y] -= 5.; if (angle[Y] <=-360) angle[Y]+=360; break;
    case 'Y': angle[Y] += 5.; if (angle[Y] >= 360) angle[Y]-=360; break;
    case 'z': angle[Z] -= 5.; if (angle[Z] <=-360) angle[Z]+=360; break;
    case 'Z': angle[Z] += 5.; if (angle[Z] >= 360) angle[Z]-=360; break;
  }
  system (CON_CLS);
  printf ("\n\r Press (keeping the GLUT window activated):");
  printf ("\n\n\r <ESC>  to quit");
  printf ("\n\n\r b         toggle Backface Culling ");
  if (backF) printf ("(ON)"); else printf ("(OFF)"); 
  printf ("\n\n\r e / E     decrease/increase Eye point z-coord.");
  printf ("\n\n\r p         toggle projection: Persp."); 
  if (persp) printf ("(ON)");
  printf ("/Orth."); if (!persp) printf ("(ON)");
  printf ("\n\n\r x / X     decrease/increase X-rotation angle");
  printf ("\n\n\r y / Y     decrease/increase Y-rotation angle");
  printf ("\n\n\r z / Z     decrease/increase Z-rotation angle");
  printf ("\n\n\r Current values: eyez    =%7.2f", eyez);
  printf ("\n\r                 angle[X]=%7.2f", angle[X]);
  printf ("\n\r                 angle[Y]=%7.2f", angle[Y]);
  printf ("\n\r                 angle[Z]=%7.2f", angle[Z]);
  draw();
  return;
}

/*************************************************************************/
  void init(void)
/*************************************************************************/
{ /*Eckpunkt-Koordinaten:*/
  Vrtx[0][X] = Vrtx[3][X] = Vrtx[4][X] = Vrtx[7][X] = -1;
  Vrtx[1][X] = Vrtx[2][X] = Vrtx[5][X] = Vrtx[6][X] =  1;

  Vrtx[0][Y] = Vrtx[1][Y] = Vrtx[4][Y] = Vrtx[5][Y] = -1;
  Vrtx[2][Y] = Vrtx[3][Y] = Vrtx[6][Y] = Vrtx[7][Y] =  1;

  Vrtx[0][Z] = Vrtx[1][Z] = Vrtx[2][Z] = Vrtx[3][Z] =  1;
  Vrtx[4][Z] = Vrtx[5][Z] = Vrtx[6][Z] = Vrtx[7][Z] = -1;

  /*Raumdiagonale des Wuerfels:*/
  diag = 3.464f; // =(2*2 + 2*2 + 2*2)^0.5
  /*Vorerst nur als Drahtmodell:*/
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  /*Tastendruck simulieren, um Menue auszugeben:*/
  key(' ', 0, 0);
}

/*************************************************************************/
  int main(int argc, char **argv)
/*************************************************************************/
{ glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE);
  glutInitWindowSize(winWidth, winHeight);
  glutCreateWindow("CG goes OpenGL");
  glutDisplayFunc(draw);
  glutKeyboardFunc(key);
  init(); 
  glutMainLoop();
  return 0;
}
/*
Zur Berechnung von viewScale:
Verkuerzung von eyez bewirkt (bei konstanten glFrustum-Parametern)
eine Oeffnung des Sichtwinkels, so dass:
winWidth(neu) / winWidth(alt) = viewScale
Da die Fensterbreiten mittig "uebereinander gelegt" werden
(winWidth(alt)/2 = x+winWidth(neu)/2), 
gilt fuer die Startkoordinate x:
x = (winWidth(alt)-winWidth(neu)) / 2
  = (winWidth(alt)-winWidth(alt)*viewScale) / 2
  = winWidth(alt)*(1-viewScale) / 2
(winHeight und y entsprechend)
max.Werte (danach Plazierung nicht mittig!?):
glViewport(-1900,-1900,4100,4100);
*/
